@@ -7,15 +7,16 @@ use Config;
 use Encode ();
 use Scalar::Util 'readonly';
 
-use constant MAX_RAND_INT => 2 ** 32;
+use constant MAX_RAND_INT       => 2**32;
 use constant MATH_RANDOM_SECURE => eval "require Math::Random::Secure;";
 
 our %TYPES = (
-    text   => 0x01,
-    binary => 0x02,
-    ping   => 0x09,
-    pong   => 0x0a,
-    close  => 0x08
+    continuation => 0x00,
+    text         => 0x01,
+    binary       => 0x02,
+    ping         => 0x09,
+    pong         => 0x0a,
+    close        => 0x08
 );
 
 sub new {
@@ -43,6 +44,10 @@ sub new {
         $self->{buffer} = $buffer;
     }
 
+    if (defined($self->{type}) && defined($TYPES{$self->{type}})) {
+        $self->opcode($TYPES{$self->{type}});
+    }
+
     $self->{version} ||= 'draft-ietf-hybi-17';
 
     $self->{fragments} = [];
@@ -79,16 +84,28 @@ sub next {
     return Encode::decode('UTF-8', $bytes);
 }
 
-sub fin    { @_ > 1 ? $_[0]->{fin}    = $_[1] : $_[0]->{fin} }
-sub rsv    { @_ > 1 ? $_[0]->{rsv}    = $_[1] : $_[0]->{rsv} }
-sub opcode { @_ > 1 ? $_[0]->{opcode} = $_[1] : $_[0]->{opcode} || 1 }
+sub fin {
+    @_ > 1 ? $_[0]->{fin} =
+        $_[1]
+      : defined($_[0]->{fin}) ? $_[0]->{fin}
+      :                         1;
+}
+sub rsv { @_ > 1 ? $_[0]->{rsv} = $_[1] : $_[0]->{rsv} }
+
+sub opcode {
+    @_ > 1 ? $_[0]->{opcode} =
+        $_[1]
+      : defined($_[0]->{opcode}) ? $_[0]->{opcode}
+      :                            1;
+}
 sub masked { @_ > 1 ? $_[0]->{masked} = $_[1] : $_[0]->{masked} }
 
-sub is_ping   { $_[0]->opcode == 9 }
-sub is_pong   { $_[0]->opcode == 10 }
-sub is_close  { $_[0]->opcode == 8 }
-sub is_text   { $_[0]->opcode == 1 }
-sub is_binary { $_[0]->opcode == 2 }
+sub is_ping         { $_[0]->opcode == 9 }
+sub is_pong         { $_[0]->opcode == 10 }
+sub is_close        { $_[0]->opcode == 8 }
+sub is_continuation { $_[0]->opcode == 0 }
+sub is_text         { $_[0]->opcode == 1 }
+sub is_binary       { $_[0]->opcode == 2 }
 
 sub next_bytes {
     my $self = shift;
@@ -146,7 +163,7 @@ sub next_bytes {
             $bits =~ s{^.}{0};
 
             # Can we handle 64bit numbers?
-            if ($Config{ivsize} <= 4 || $Config{longsize} < 8) {
+            if ($Config{ivsize} <= 4 || $Config{longsize} < 8 || $] < 5.010) {
                 $bits = substr($bits, 32);
                 $payload_len = unpack 'N', pack 'B*', $bits;
             }
@@ -159,8 +176,9 @@ sub next_bytes {
 
         if ($payload_len > $self->{max_payload_size}) {
             $self->{buffer} = '';
-            die
-              "Payload is too big. Deny big message ($payload_len) or increase max_payload_size ($self->{max_payload_size})";
+            die "Payload is too big. "
+              . "Deny big message ($payload_len) "
+              . "or increase max_payload_size ($self->{max_payload_size})";
         }
 
         my $mask;
@@ -229,21 +247,15 @@ sub to_bytes {
     }
 
     if (length $self->{buffer} > $self->{max_payload_size}) {
-        die
-          "Payload is too big. Send shorter messages or increase max_payload_size";
+        die "Payload is too big. "
+          . "Send shorter messages or increase max_payload_size";
     }
 
     my $string = '';
 
-    my $opcode;
-    if (my $type = $self->{type}) {
-        $opcode = $TYPES{$type};
-    }
-    else {
-        $opcode = $self->opcode || 1;
-    }
+    my $opcode = $self->opcode;
 
-    $string .= pack 'C', ($opcode + 128);
+    $string .= pack 'C', ($opcode + ($self->fin ? 128 : 0));
 
     my $payload_len = length($self->{buffer});
     if ($payload_len <= 125) {
@@ -287,12 +299,6 @@ sub to_string {
     my $self = shift;
 
     die 'DO NOT USE';
-
-    if (   $self->version eq 'draft-hixie-75'
-        || $self->version eq 'draft-ietf-hybi-00')
-    {
-        return "\x00" . Encode::decode('UTF-8', $self->{buffer}) . "\xff";
-    }
 }
 
 sub _mask {
@@ -301,7 +307,7 @@ sub _mask {
 
     $mask = $mask x (int(length($payload) / 4) + 1);
     $mask = substr($mask, 0, length($payload));
-    $payload ^= $mask;
+    $payload = "$payload" ^ $mask;
 
     return $payload;
 }
@@ -334,26 +340,57 @@ Construct or parse a WebSocket frame.
 By default built-in C<rand> is used, this is not secure, so when
 L<Math::Random::Secure> is installed it is used instead.
 
-=head1 ATTRIBUTES
+=head1 METHODS
 
-=head2 C<type>
+=head2 C<new>
 
-Frame's type. C<text> by default. Other accepted values:
+    Protocol::WebSocket::Frame->new('data');   # same as (buffer => 'data')
+    Protocol::WebSocket::Frame->new(buffer => 'data', type => 'close');
 
+Create a new L<Protocol::WebSocket::Frame> instance. Automatically detect if the
+passed data is a Perl string or bytes.
+
+When called with more than one arguments, it takes the following named arguments
+(all of them are optional).
+
+=over
+
+=item C<buffer> => STR (default: C<"">)
+
+The payload of the frame.
+
+=item C<type> => TYPE_STR (default: C<"text">)
+
+The type of the frame. Accepted values are:
+
+    continuation
+    text
     binary
     ping
     pong
     close
 
-=head1 METHODS
+=item C<opcode> => INT (default: 1)
 
-=head2 C<new>
+The opcode of the frame. If C<type> field is set to a valid string, this field is ignored.
 
-    Protocol::WebSocket::Frame->new('data');
-    Protocol::WebSocket::Frame->new(buffer => 'data', type => 'close');
+=item C<fin> => BOOL (default: 1)
 
-Create a new L<Protocol::WebSocket::Frame> instance. Automatically detect if the
-passed data is a Perl string or bytes.
+"fin" flag of the frame. "fin" flag must be 1 in the ending frame of fragments.
+
+=item C<masked> => BOOL (default: 0)
+
+If set to true, the frame will be masked.
+
+=item C<version> => VERSION_STR (default: C<'draft-ietf-hybi-17'>)
+
+WebSocket protocol version string. See L<Protocol::WebSocket> for valid version strings.
+
+=back
+
+=head2 C<is_continuation>
+
+Check if frame is of continuation type.
 
 =head2 C<is_text>
 
@@ -375,12 +412,22 @@ Check if frame is a pong response.
 
 Check if frame is of close type.
 
+=head2 C<opcode>
+
+    $opcode = $frame->opcode;
+    $frame->opcode(8);
+
+Get/set opcode of the frame.
+
 =head2 C<append>
 
-    $frame->append(...);
+    $frame->append($chunk);
 
 Append a frame chunk.
 
+Beware that this method is B<destructive>.
+It makes C<$chunk> empty unless C<$chunk> is read-only.
+
 =head2 C<next>
 
     $frame->append(...);
